@@ -14,7 +14,7 @@ class Base(Device):
         Device.__init__(self, 'base')
         self.left_wheel = Stepper(usb='/dev/hello-motor-left-wheel')
         self.right_wheel = Stepper(usb='/dev/hello-motor-right-wheel')
-        self.status = {'timestamp_pc':0,'x':0,'y':0,'theta':0,'x_vel':0,'y_vel':0,'theta_vel':0, 'pose_time_s':0,'effort': [0, 0], 'left_wheel': self.left_wheel.status, 'right_wheel': self.right_wheel.status}
+        self.status = {'timestamp_pc':0,'x':0,'y':0,'theta':0,'x_vel':0,'y_vel':0,'theta_vel':0, 'pose_time_s':0,'effort': [0, 0], 'left_wheel': self.left_wheel.status, 'right_wheel': self.right_wheel.status, 'translation_force': 0, 'rotation_torque': 0}
         self.first_step=True
         wheel_circumference_m = self.params['wheel_diameter_m'] * pi
         self.meters_per_motor_rad = (wheel_circumference_m / (2.0 * pi)) / self.params['gr']
@@ -29,8 +29,9 @@ class Base(Device):
     # ###########  Device Methods #############
 
     def startup(self):
-        return self.left_wheel.startup() and self.right_wheel.startup()
-
+        success=self.left_wheel.startup() and self.right_wheel.startup()
+        self.__update_status()
+        return success
 
     def stop(self):
         self.left_wheel.stop()
@@ -288,6 +289,9 @@ class Base(Device):
         """
         self.left_wheel.pull_status()
         self.right_wheel.pull_status()
+        self.__update_status()
+
+    def __update_status(self):
         self.status['timestamp_pc'] = time.time()
 
         p0 = self.status['left_wheel']['pos']
