@@ -4,6 +4,49 @@ from stretch_body.device import Device
 import time
 from stretch_body.hello_utils import *
 import termios
+import numpy
+
+class DynamixelCommErrorStats(Device):
+    def __init__(self, name, logger):
+        Device.__init__(self, name='dxl_comm_errors')
+        self.name=name
+        self.status={'n_rx':0, 'n_tx':0, 'n_gsr': 0, 'error_rate_avg_hz':0}
+        self.rate_log=None
+        self.n_log=10
+        self.log_idx =0
+        self.ts_error_last=time.time()
+        self.ts_warn_last=time.time()
+        self.logger=logger
+
+    def add_error(self,rx=True, gsr=False):
+        t = time.time()
+        if type(self.rate_log)==type(None): #First error
+            self.rate_log=numpy.array([0.0] * self.n_log)
+        self.rate_log[self.log_idx]=1/(t-self.ts_error_last)
+        self.log_idx = (self.log_idx + 1) % self.n_log
+        self.status['error_rate_avg_hz'] = numpy.average(self.rate_log)
+        if rx:
+            self.status['n_rx']+=1
+        else:
+            self.status['n_tx'] += 1
+        if gsr:
+            self.status['n_gsr'] += 1
+        if t-self.ts_warn_last>self.params['warn_every_s']:
+            self.ts_warn_last=t
+            if self.status['error_rate_avg_hz']>self.params['warn_above_rate']:
+                self.logger.warning('Device %s generating %f errors per minute'%(self.name,(self.status['error_rate_avg_hz']*60)))
+        if self.params['verbose']:
+            self.pretty_print()
+
+    def pretty_print(self):
+        print('---- Dynamixel Comm Errors %s ----'%self.name)
+        print('Rate (Hz): %f' % self.status['error_rate_avg_hz'])
+        print('Rate (errors per minute): %f'%(self.status['error_rate_avg_hz']*60))
+        print('Num TX: %f'%self.status['n_tx'])
+        print('Num RX: %f' % self.status['n_rx'])
+        print('Num Group Sync RX: %f' % self.status['n_gsr'])
+
+
 
 class DynamixelHelloXL430(Device):
     """
@@ -27,67 +70,105 @@ class DynamixelHelloXL430(Device):
         self.ts_over_eff_start=None
         self.hw_valid=False
         self.is_calibrated=False
-        self.set_soft_motion_limits(None, None)
-        self.is_homing=False
 
-    # ###########  Device Methods #############
-    def set_soft_motion_limits(self,x_min=None,x_max=None):
+
         if self.params['flip_encoder_polarity']:
             wr_max = self.ticks_to_world_rad(self.params['range_t'][0])
             wr_min = self.ticks_to_world_rad(self.params['range_t'][1])
         else:
             wr_max = self.ticks_to_world_rad(self.params['range_t'][1])
             wr_min = self.ticks_to_world_rad(self.params['range_t'][0])
+        self.soft_motion_limits = {'collision': [None, None], 'user': [None, None],'hard': [wr_min,wr_max],'current': [wr_min,wr_max]}
 
-        if x_min is not None:
-            x_min=max(x_min,wr_min)
-        else:
-            x_min=wr_min
-        if x_max is not None:
-            x_max=min(x_max,wr_max)
-        else:
-            x_max=wr_max
-        self.soft_motion_limits=[x_min,x_max]
+        self.is_homing=False
+        self.status_mux_id = 0
+        self.was_runstopped = False
+        self.comm_errors = DynamixelCommErrorStats(name,logger=self.logger)
+
+    # ###########  Device Methods #############
+
+    def get_soft_motion_limits(self):
+        """
+            Return the currently applied soft motion limits: [min, max]
+
+            The soft motion limit restricts joint motion to be <= its physical limits.
+
+            There are three types of limits:
+            Hard: The physical limits
+            Collision: Limits set by RobotCollision to avoid collisions
+            User: Limits set by the user software
+
+            The joint is limited to the most restrictive range of the Hard / Collision/ User values.
+            Specifying a value of None for a limit indicates that no constraint exists for that limit type.
+            This allows a User limits and Collision limits to coexist.
+            For example, a user can temporarily restrict the range of motion beyond the current collision limits.
+            Then, by commanding User limits of None, the joint limits will revert back to the collision settings.
+        """
+        return self.soft_motion_limits['current']
+
+    def set_soft_motion_limit_min(self,x,limit_type='user' ):
+        """
+        x: value to set a joints limit to
+        limit_type: 'user' or 'collision'
+        """
+        self.soft_motion_limits[limit_type][0]=x
+        self.soft_motion_limits['current'][0]=max(filter(lambda x: x is not None, [self.soft_motion_limits['collision'][0],self.soft_motion_limits['hard'][0],self.soft_motion_limits['user'][0]]))
+
+
+    def set_soft_motion_limit_max(self,x,limit_type='user' ):
+        """
+        x: value to set a joints limit to
+        limit_type: 'user' or 'collision'
+        """
+        self.soft_motion_limits[limit_type][1]=x
+        self.soft_motion_limits['current'][1]=min(filter(lambda x: x is not None, [self.soft_motion_limits['collision'][1],self.soft_motion_limits['hard'][1],self.soft_motion_limits['user'][1]]))
+
+    # ###################################################
 
     def do_ping(self, verbose=False):
         return self.motor.do_ping(verbose)
 
     def startup(self):
-        if self.motor.do_ping(verbose=False):
-            self.hw_valid = True
-            self.motor.disable_torque()
-            if self.params['use_multiturn']:
-                self.motor.enable_multiturn()
+        try:
+            if self.motor.do_ping(verbose=False):
+                self.hw_valid = True
+                self.motor.disable_torque()
+                if self.params['use_multiturn']:
+                    self.motor.enable_multiturn()
+                else:
+                    self.motor.enable_pos()
+                    if self.params['range_t'][0]<0 or self.params['range_t'][1]>4095:
+                        self.logger.warning('Warning: Invalid position range for %s'%self.name)
+                self.motor.set_pwm_limit(self.params['pwm_limit'])
+                self.motor.set_temperature_limit(self.params['temperature_limit'])
+                self.motor.set_min_voltage_limit(self.params['min_voltage_limit'])
+                self.motor.set_max_voltage_limit(self.params['max_voltage_limit'])
+                self.motor.set_P_gain(self.params['pid'][0])
+                self.motor.set_I_gain(self.params['pid'][1])
+                self.motor.set_D_gain(self.params['pid'][2])
+                self.motor.set_return_delay_time(self.params['return_delay_time'])
+                self.motor.set_profile_velocity(self.rad_per_sec_to_ticks(self.params['motion']['default']['vel']))
+                self.motor.set_profile_acceleration(self.rad_per_sec_sec_to_ticks(self.params['motion']['default']['accel']))
+                self.v_des=self.params['motion']['default']['vel']
+                self.a_des=self.params['motion']['default']['accel']
+                self.is_calibrated=self.motor.is_calibrated()
+                self.enable_torque()
+                self.pull_status()
+                return True
             else:
-                self.motor.enable_pos()
-                if self.params['range_t'][0]<0 or self.params['range_t'][1]>4095:
-                    self.logger.warn('Warning: Invalid position range for %s'%self.name)
-            self.motor.set_pwm_limit(self.params['pwm_limit'])
-            self.motor.set_temperature_limit(self.params['temperature_limit'])
-            self.motor.set_min_voltage_limit(self.params['min_voltage_limit'])
-            self.motor.set_max_voltage_limit(self.params['max_voltage_limit'])
-            self.motor.set_P_gain(self.params['pid'][0])
-            self.motor.set_I_gain(self.params['pid'][1])
-            self.motor.set_D_gain(self.params['pid'][2])
-            self.motor.set_return_delay_time(self.params['return_delay_time'])
-            self.motor.set_profile_velocity(self.rad_per_sec_to_ticks(self.params['motion']['default']['vel']))
-            self.motor.set_profile_acceleration(self.rad_per_sec_sec_to_ticks(self.params['motion']['default']['accel']))
-            self.v_des=self.params['motion']['default']['vel']
-            self.a_des=self.params['motion']['default']['accel']
-            self.set_range()
-            self.is_calibrated=self.motor.is_calibrated()
-            self.enable_torque()
-            return True
-        else:
-            self.logger.warn('DynamixelHelloXL430 Ping failed... %s' % self.name)
+                self.logger.warning('DynamixelHelloXL430 Ping failed... %s' % self.name)
+                print('DynamixelHelloXL430 Ping failed...', self.name)
+                return False
+        except DynamixelCommError:
+            self.logger.warning('DynamixelHelloXL430 Ping failed... %s' % self.name)
             print('DynamixelHelloXL430 Ping failed...', self.name)
             return False
 
 
     def stop(self):
         if self.hw_valid:
-            self.hw_valid = False
             self.disable_torque()
+            self.hw_valid = False
 
     def pull_status(self,data=None):
         if not self.hw_valid:
@@ -113,28 +194,41 @@ class DynamixelHelloXL430(Device):
                     v = self.motor.get_vel()
                 vel_valid = self.motor.last_comm_success
 
-                eff = self.motor.get_load()
-                if not self.motor.last_comm_success and self.params['retry_on_comm_failure']:
+                if self.status_mux_id == 0:
                     eff = self.motor.get_load()
-                eff_valid = self.motor.last_comm_success
+                    if not self.motor.last_comm_success and self.params['retry_on_comm_failure']:
+                        eff = self.motor.get_load()
+                    eff_valid = self.motor.last_comm_success
+                else:
+                    eff = self.status['effort_ticks']
 
-                temp = self.motor.get_temp()
-                if not self.motor.last_comm_success and self.params['retry_on_comm_failure']:
+                if self.status_mux_id == 1:
                     temp = self.motor.get_temp()
-                temp_valid = self.motor.last_comm_success
+                    if not self.motor.last_comm_success and self.params['retry_on_comm_failure']:
+                        temp = self.motor.get_temp()
+                    temp_valid = self.motor.last_comm_success
+                else:
+                    temp = self.status['temp']
 
-                err=self.motor.get_hardware_error()
-                if not self.motor.last_comm_success and self.params['retry_on_comm_failure']:
-                    err=self.motor.get_hardware_error()
-                err_valid = self.motor.last_comm_success
+                if self.status_mux_id == 2:
+                    err = self.motor.get_hardware_error()
+                    if not self.motor.last_comm_success and self.params['retry_on_comm_failure']:
+                        err = self.motor.get_hardware_error()
+                    err_valid = self.motor.last_comm_success
+                else:
+                    err = self.status['hardware_error']
+
+                self.status_mux_id = (self.status_mux_id + 1) % 3
 
 
                 if not pos_valid or not vel_valid or not eff_valid or not temp_valid or not err_valid:
-                    self.logger.debug('Failed status communication on %s: POS %d VEL %d EFF %d TEMP %d ERR %d '%(self.name,pos_valid,vel_valid,eff_valid,temp_valid,err_valid))
-
+                    raise DynamixelCommError
                 ts = time.time()
-            except termios.error:
-                self.logger.error('Dynamixel communication error on %s: '%self.name)
+            except(termios.error, DynamixelCommError):
+                #self.logger.warning('Dynamixel communication error on %s: '%self.name)
+                self.port_handler.ser.reset_output_buffer()
+                self.port_handler.ser.reset_input_buffer()
+                self.comm_errors.add_error(rx=True,gsr=False)
                 return
         else:
             x = data['x']
@@ -175,7 +269,7 @@ class DynamixelHelloXL430(Device):
         if self.status['stalled']:
             if not over_eff:
                 self.ts_over_eff_start = None
-            if over_eff and self.ts_over_eff_start is None: #Mark the start of being stall and over-effort
+            if over_eff and self.ts_over_eff_start is None: #Mark the start of being stalled and over-effort
                 self.ts_over_eff_start = time.time()
             if self.ts_over_eff_start is not None and time.time()-self.ts_over_eff_start>self.params['stall_max_time']:
                 self.status['stall_overload'] = True
@@ -225,6 +319,17 @@ class DynamixelHelloXL430(Device):
         print('Is Calibrated',self.is_calibrated)
         #self.motor.pretty_print()
 
+    def step_sentry(self, robot):
+
+        if self.hw_valid and self.robot_params['robot_sentry']['dynamixel_stop_on_runstop'] and self.params['enable_runstop']:
+            is_runstopped = robot.pimu.status['runstop_event']
+            if is_runstopped is not self.was_runstopped:
+                if is_runstopped:
+                    self.disable_torque()
+                else:
+                    self.enable_torque()
+            self.was_runstopped = is_runstopped
+
     # #####################################
 
     def reboot(self):
@@ -246,90 +351,101 @@ class DynamixelHelloXL430(Device):
         if not self.hw_valid:
             return
         if self.params['req_calibration'] and not self.is_calibrated:
-            self.logger.warn('Dynamixel not calibrated: %s' % self.name)
+            self.logger.warning('Dynamixel not calibrated: %s' % self.name)
             print('Dynamixel not calibrated:', self.name)
             return
         try:
             self.set_motion_params(v_des,a_des)
-            x_des = min(max(self.soft_motion_limits[0], x_des), self.soft_motion_limits[1])
+            x_des = min(max(self.get_soft_motion_limits()[0], x_des), self.get_soft_motion_limits()[1])
             t_des = self.world_rad_to_ticks(x_des)
             t_des = max(self.params['range_t'][0], min(self.params['range_t'][1], t_des))
             self.motor.go_to_pos(t_des)
-        except termios.error:
-            self.logger.error('Dynamixel communication error at time: %f' % time.time())
+        except (termios.error, DynamixelCommError):
+            #self.logger.warning('Dynamixel communication error on: %s' % self.name)
+            self.comm_errors.add_error(rx=False, gsr=False)
 
 
-    def set_range(self,x_min=None,x_max=None):
-        if x_min is not None:
-            t_min=self.world_rad_to_ticks(x_min)
-            t_min=max(t_min,self.params['range_t'][0])
-        else:
-            t_min=self.params['range_t'][0]
-        if x_max is not None:
-            t_max=self.world_rad_to_ticks(x_max)
-            t_max=max(t_max,self.params['range_t'][1])
-        else:
-            t_max=self.params['range_t'][1]
-        self.range=[t_min,t_max]
-
     def set_motion_params(self,v_des=None,a_des=None):
-        if not self.hw_valid:
-            return
-        if v_des is not None:
-            v_des = min(self.params['motion']['max']['vel'], v_des)
+        try:
+            if not self.hw_valid:
+                return
 
+            v_des = v_des if v_des is not None else self.params['motion']['default']['vel']
+            v_des = min(self.params['motion']['max']['vel'], v_des)
             if v_des != self.v_des:
                 self.motor.set_profile_velocity(self.rad_per_sec_to_ticks(v_des))
                 self.v_des = v_des
-        if a_des is not None:
+
+            a_des = a_des if a_des is not None else self.params['motion']['default']['accel']
             a_des = min(self.params['motion']['max']['accel'], a_des)
             if a_des != self.a_des:
                 self.motor.set_profile_acceleration(self.rad_per_sec_sec_to_ticks(a_des))
                 self.a_des = a_des
+        except (termios.error, DynamixelCommError):
+            #self.logger.warning('Dynamixel communication error on: %s' % self.name)
+            self.comm_errors.add_error(rx=False, gsr=False)
+
 
     def move_by(self,x_des, v_des=None, a_des=None):
         if not self.hw_valid:
             return
-        if abs(x_des) > 0.00002: #Avoid drift
-            x=self.motor.get_pos()
-            if not self.motor.last_comm_success and self.params['retry_on_comm_failure']:
-                x = self.motor.get_pos()
+        try:
+            if abs(x_des) > 0.00002: #Avoid drift
+                x=self.motor.get_pos()
+                if not self.motor.last_comm_success and self.params['retry_on_comm_failure']:
+                    x = self.motor.get_pos()
 
-            if self.motor.last_comm_success:
-                cx=self.ticks_to_world_rad(x)
-                self.move_to(cx + x_des, v_des, a_des)
-            else:
-                self.logger.debug('Move_By comm failure on %s' % self.name)
+                if self.motor.last_comm_success:
+                    cx=self.ticks_to_world_rad(x)
+                    self.move_to(cx + x_des, v_des, a_des)
+                else:
+                    self.logger.debug('Move_By comm failure on %s' % self.name)
+        except (termios.error, DynamixelCommError):
+            #self.logger.warning('Dynamixel communication error on: %s' % self.name)
+            self.comm_errors.add_error(rx=False, gsr=False)
 
     def quick_stop(self):
         if not self.hw_valid:
             return
-        self.motor.disable_torque()
-        self.motor.enable_torque()
+        try:
+            self.motor.disable_torque()
+            self.motor.enable_torque()
+        except (termios.error, DynamixelCommError):
+            self.comm_errors.add_error(rx=False, gsr=False)
 
     def enable_pos(self):
         if not self.hw_valid:
             return
-        self.motor.disable_torque()
-        if self.params['use_multiturn']:
-            self.motor.enable_multiturn()
-        else:
-            self.motor.enable_pos()
-        self.motor.set_profile_velocity(self.rad_per_sec_to_ticks(self.v_des))
-        self.motor.set_profile_acceleration(self.rad_per_sec_sec_to_ticks(self.a_des))
-        self.motor.enable_torque()
+        try:
+            self.motor.disable_torque()
+            if self.params['use_multiturn']:
+                self.motor.enable_multiturn()
+            else:
+                self.motor.enable_pos()
+            self.motor.set_profile_velocity(self.rad_per_sec_to_ticks(self.v_des))
+            self.motor.set_profile_acceleration(self.rad_per_sec_sec_to_ticks(self.a_des))
+            self.motor.enable_torque()
+        except (termios.error, DynamixelCommError):
+            self.comm_errors.add_error(rx=False, gsr=False)
 
     def enable_pwm(self):
         if not self.hw_valid:
             return
-        self.motor.disable_torque()
-        self.motor.enable_pwm()
-        self.motor.enable_torque()
+        try:
+            self.motor.disable_torque()
+            self.motor.enable_pwm()
+            self.motor.enable_torque()
+        except (termios.error, DynamixelCommError):
+            self.comm_errors.add_error(rx=False, gsr=False)
+
 
     def set_pwm(self,x):
         if not self.hw_valid:
             return
-        self.motor.set_pwm(x)
+        try:
+            self.motor.set_pwm(x)
+        except (termios.error, DynamixelCommError):
+            self.comm_errors.add_error(rx=False, gsr=False)
 
 # ##########################################
     """
@@ -370,7 +486,7 @@ class DynamixelHelloXL430(Device):
         # Second hardstop is optional
 
         if not self.hw_valid:
-            self.logger.warn('Not able to home %s. Hardware not present'%self.name)
+            self.logger.warning('Not able to home %s. Hardware not present'%self.name)
             return
         if not self.params['req_calibration']:
             print('Homing not required for: '+self.name)
@@ -378,7 +494,7 @@ class DynamixelHelloXL430(Device):
 
         self.pull_status()
         if self.status['overload_error'] or self.status['overheating_error']:
-            self.logger.warn('Hardware error, unable to home. Exiting')
+            self.logger.warning('Hardware error, unable to home. Exiting')
             return
 
         self.is_homing=True
@@ -396,10 +512,10 @@ class DynamixelHelloXL430(Device):
         self.set_pwm(0)
 
         if timeout:
-            self.logger.warn('Timed out moving to first hardstop. Exiting.')
+            self.logger.warning('Timed out moving to first hardstop. Exiting.')
             return
         if self.status['overload_error'] or self.status['overheating_error']:
-            self.logger.warn('Hardware error, unable to home. Exiting')
+            self.logger.warning('Hardware error, unable to home. Exiting')
             return
 
         print('Contact at position: %d'%self.motor.get_pos())
@@ -430,10 +546,10 @@ class DynamixelHelloXL430(Device):
             self.set_pwm(0)
 
             if timeout:
-                self.logger.warn('Timed out moving to second hardstop. Exiting.')
+                self.logger.warning('Timed out moving to second hardstop. Exiting.')
                 return
             if self.status['overload_error'] or self.status['overheating_error']:
-                self.logger.warn('Hardware error, unable to home. Exiting')
+                self.logger.warning('Hardware error, unable to home. Exiting')
                 return
 
             x_dir_1 = self.motor.get_pos()
@@ -472,6 +588,12 @@ class DynamixelHelloXL430(Device):
         t= self.rad_to_ticks(rad_servo)
         return t+self.params['zero_t']
 
+    def world_rad_to_ticks_per_sec(self,r):
+        rad_per_sec_servo = r*self.params['gr']*self.polarity
+        t= self.rad_per_sec_to_ticks(rad_per_sec_servo)
+        return t
+
+
     def ticks_to_rad(self,t):
         return deg_to_rad((360.0 * t / 4096.0))
 
