@@ -133,7 +133,7 @@ class Pimu(Device):
         self.status = {'voltage': 0, 'current': 0, 'temp': 0,'cpu_temp': 0, 'cliff_range':[0,0,0,0], 'frame_id': 0,
                        'timestamp': 0,'at_cliff':[False,False,False,False], 'runstop_event': False, 'bump_event_cnt': 0,
                        'cliff_event': False, 'fan_on': False, 'buzzer_on': False, 'low_voltage_alert':False,'high_current_alert':False,'over_tilt_alert':False,
-                       'imu': self.imu.status,'debug':0,'state':0,
+                       'imu': self.imu.status,'debug':0,'state':0,'motor_sync_drop':0,
                        'transport': self.transport.status}
         self._trigger=0
         self.ts_last_fan_on=None
@@ -146,6 +146,9 @@ class Pimu(Device):
         self.board_info = {'board_version': None, 'firmware_version': None, 'protocol_version': None}
         self.valid_firmware_protocol = 'p0'
         self.hw_valid = False
+        self.ts_last_motor_sync=None
+        self.ts_last_motor_sync_warn=None
+
 
     # ###########  Device Methods #############
 
@@ -168,7 +171,7 @@ class Pimu(Device):
                     Please upgrade the firmware and/or version of Stretch Body.
                     ----------------
                     """.format(self.name, self.board_info['protocol_version'], self.valid_firmware_protocol)
-                    self.logger.warn(textwrap.dedent(protocol_msg))
+                    self.logger.warning(textwrap.dedent(protocol_msg))
                     self.hw_valid=False
                     self.transport.stop()
 
@@ -182,10 +185,10 @@ class Pimu(Device):
         if not self.hw_valid:
             return
         with self.lock:
-            self.hw_valid = False
             self.set_fan_off()
             self.push_command(exiting=True)
             self.transport.stop()
+            self.hw_valid = False
 
     def pull_status(self,exiting=False):
         if not self.hw_valid:
@@ -234,6 +237,7 @@ class Pimu(Device):
         print('Debug', self.status['debug'])
         print('Timestamp', self.status['timestamp'])
         print('Read error', self.transport.status['read_error'])
+        print('Dropped motor sync',self.status['motor_sync_drop'])
         print('Board version:',self.board_info['board_version'])
         print('Firmware version:', self.board_info['firmware_version'])
         self.imu.pretty_print()
@@ -274,10 +278,20 @@ class Pimu(Device):
         #Push out immediately
         if not self.hw_valid:
             return
+
+        t = time.time()
+        if self.ts_last_motor_sync is not None and t-self.ts_last_motor_sync<1.0/self.params['max_sync_rate_hz']:
+            self.status['motor_sync_drop'] += 1
+            if self.ts_last_motor_sync_warn is None or t-self.ts_last_motor_sync_warn>5.0:
+                print('Warning: Rate of calls to Pimu:trigger_motor_sync above maximum frequency of %.2f Hz. Motor commands dropped: %d'%(self.params['max_sync_rate_hz'],self.status['motor_sync_drop']))
+                self.ts_last_motor_sync_warn=t
+            return
+
         with self.lock:
             self.transport.payload_out[0] = RPC_SET_MOTOR_SYNC
             self.transport.queue_rpc(1, self.rpc_motor_sync_reply)
             self.transport.step()
+            self.ts_last_motor_sync = t
 
     def set_fan_on(self):
         with self.lock:
@@ -414,21 +428,21 @@ class Pimu(Device):
 
     def rpc_motor_sync_reply(self,reply):
         if reply[0] != RPC_REPLY_MOTOR_SYNC:
-            print('Error RPC_REPLY_MOTOR_SYNC', reply[0])
+            self.logger.warning('Error RPC_REPLY_MOTOR_SYNC', reply[0])
 
     def rpc_config_reply(self,reply):
         if reply[0] != RPC_REPLY_PIMU_CONFIG:
-            print('Error RPC_REPLY_PIMU_CONFIG', reply[0])
+            self.logger.warning('Error RPC_REPLY_PIMU_CONFIG', reply[0])
 
     def rpc_board_info_reply(self,reply):
         if reply[0] == RPC_REPLY_PIMU_BOARD_INFO:
             self.unpack_board_info(reply[1:])
         else:
-            print('Error RPC_REPLY_PIMU_BOARD_INFO', reply[0])
+            self.logger.warning('Error RPC_REPLY_PIMU_BOARD_INFO', reply[0])
 
     def rpc_trigger_reply(self,reply):
         if reply[0] != RPC_REPLY_PIMU_TRIGGER:
-            print('Error RPC_REPLY_PIMU_TRIGGER', reply[0])
+            self.logger.warning('Error RPC_REPLY_PIMU_TRIGGER', reply[0])
         else:
             tt=unpack_uint32_t(reply[1:])
 
@@ -436,17 +450,22 @@ class Pimu(Device):
         if reply[0] == RPC_REPLY_PIMU_STATUS:
             self.unpack_status(reply[1:])
         else:
-            print('Error RPC_REPLY_PIMU_STATUS', reply[0])
+            self.logger.warning('Error RPC_REPLY_PIMU_STATUS', reply[0])
 
 
     # ################ Sentry #####################
     def get_cpu_temp(self):
         cpu_temp = 0
-        t = psutil.sensors_temperatures()['coretemp']
-        for c in t:
-            cpu_temp = max(cpu_temp, c.current)
+        try:
+            t = psutil.sensors_temperatures()['coretemp']
+            for c in t:
+                cpu_temp = max(cpu_temp, c.current)
+        except(KeyError, IOError): #May not be available on virtual machines
+            cpu_temp=25.0
         return cpu_temp
 
+
+
     def step_sentry(self):
         if self.hw_valid and self.robot_params['robot_sentry']['base_fan_control']:
             #Manage CPU temp using the mobile base fan
