@@ -71,7 +71,7 @@ class Wacc(Device):
                     Please upgrade the firmware and/or version of Stretch Body.
                     ----------------
                     """.format(self.name, self.board_info['protocol_version'], self.valid_firmware_protocol)
-                    self.logger.warn(textwrap.dedent(protocol_msg))
+                    self.logger.warning(textwrap.dedent(protocol_msg))
                     self.hw_valid=False
                     self.transport.stop()
 
@@ -87,9 +87,9 @@ class Wacc(Device):
         if not self.hw_valid:
             return
         with self.lock:
-            self.hw_valid = False
             self.push_command(exiting=True)
             self.transport.stop()
+            self.hw_valid = False
 
     def set_D2(self,on):#0 or 1
         """
@@ -219,21 +219,21 @@ class Wacc(Device):
         if reply[0] == RPC_REPLY_WACC_BOARD_INFO:
             self.unpack_board_info(reply[1:])
         else:
-            print('Error RPC_REPLY_WACC_BOARD_INFO', reply[0])
+            self.logger.warning('Error RPC_REPLY_WACC_BOARD_INFO', reply[0])
 
     def rpc_command_reply(self,reply):
         if reply[0] != RPC_REPLY_WACC_COMMAND:
-            print('Error RPC_REPLY_WACC_COMMAND', reply[0])
+            self.logger.warning('Error RPC_REPLY_WACC_COMMAND', reply[0])
 
     def rpc_config_reply(self,reply):
         if reply[0] != RPC_REPLY_WACC_CONFIG:
-            print('Error RPC_REPLY_WACC_CONFIG', reply[0])
+            self.logger.warning('Error RPC_REPLY_WACC_CONFIG', reply[0])
 
     def rpc_status_reply(self,reply):
         if reply[0] == RPC_REPLY_WACC_STATUS:
             self.unpack_status(reply[1:])
         else:
-            print('Error RPC_REPLY_WACC_STATUS', reply[0])
+            self.logger.warning('Error RPC_REPLY_WACC_STATUS', reply[0])
 
 
 
