@@ -10,13 +10,24 @@ import time
 
 class TestPimu(unittest.TestCase):
 
+    def test_motor_sync_rate(self):
+        print('test_motor_sync_rate')
+        p = stretch_body.pimu.Pimu()
+        p.startup()
+        for i in range(100):
+            p.trigger_motor_sync()
+            time.sleep(1/p.params['max_sync_rate_hz'])
+        self.assertTrue(p.status['motor_sync_drop']==0)
+        for i in range(100):
+            p.trigger_motor_sync()
+            time.sleep(0.01)
+        self.assertTrue(p.status['motor_sync_drop']>75)
+
     def test_invalid_protocol(self):
         """Simulate an invalid protocol and verify the correct error.
         """
         p = stretch_body.pimu.Pimu()
         p.valid_firmware_protocol = 'p-1' # valid protocols are p0 and up
-        p.startup()
-        # TODO: capture logging with https://testfixtures.readthedocs.io/en/latest/logging.html
-        #       verify output is '[WARNING] [pimu]: \n----------------\nFirmware protocol mismatch on hello-pimu. [...]'
+        self.assertFalse(p.startup())
 
         p.stop()
